%% Sintonía de Control PID via CHR
% Sergio Andres Castaño Giraldo
% https://controlautomaticoeducacion.com/
clc
clear
close all

%% Proceso Real
G=tf([2],conv([2 1],[5 1]));
G.iodelay=1;

%% Identificacion del Modelo via Ziegler y Nichols
%tiempo
dt=0.01;
t=0:dt:35;
%Entrada
u(1:length(t))=1;
%Salida
y=lsim(G,u,t);
dy=diff(y)/dt; %Derivada
dy2=diff(dy)/dt;

%Encontrar el punto de inflexion y su derivada
% el punto donde la pendiente de la respuesta escalón tiene su valor máximo (punto de inflexión)
[m,p]=max(dy);
yp=y(p);
tp=t(p);
tm=0:20; 
ym=m*(tm-tp)+yp; %Ecuacion de la recta

%Grafica punto de inflexión
figure
plot([t(1) t(end)],[2 2],'--k',[11 11],[0 2],'--k','linewidth',2);
hold on
plot(t,y,tp,yp,'o',tm,ym,'-r','linewidth',3);
axis([0 35 0 3]); 
box off
ylabel('$$c(t)$$','FontSize',20,'Interpreter','latex')
xlabel('$$t$$','FontSize',20,'Interpreter','latex')
set(gca,'FontSize',(20) )

%% Modelo Identificado del Sistema
k = 2;    %Ganancia del Sistema
theta=2;  %Retardo
tau=9; %Constante de Tiempo
%Modelo
Gm=tf([k],[tau 1]);
Gm.iodelay=theta;

%% Comparar Sistema Real vs Modelo
figure
step(G,Gm)
legend('Planta','Modelo')

%% Método 1 CHR
% sintonía propuesta por el método CHR para el criterio de desempeño 
% "respuesta más rápida posible sin sobreimpulso" suponiendo que el sistema
% es implementado como tipo servo siguiendo una referencia.

% Control P - CHR
kc1 = (0.3*tau) / (k*theta);
%Ganancias
Kp1 = kc1;

% Control PI - CHR
kc2 = (0.35*tau) / (k*theta);
ti2 = 1.16*tau;
%Ganancias
Kp2 = kc2;
Ki2 = kc2/ti2;

% Control PID - CHR
kc3 = (0.6*tau) / (k*theta);
ti3 = tau;
td3 = theta/2;
%Ganancias
Kp3 = kc3;
Ki3 = kc3/ti3;
Kd3 = kc3*td3;

%% Método 2 CHR
% sintonía propuesta por el método CHR para el criterio de desempeño 
% "respuesta más rápida posible sin sobreimpulso" suponiendo que el sistema 
% es implementado como un control regulatório (perturbación de carga tipo 
% escalón). Note que esta sintonia es más agresiva que la sintonia anterior.

% Control P - CHR
kc4 = (0.3*tau) / (k*theta);
%Ganancias
Kp4 = kc4;

% Control PI - CHR
kc5 = (0.6*tau) / (k*theta);
ti5 = 4*theta;
%Ganancias
Kp5 = kc5;
Ki5 = kc5/ti5;

% Control PID - CHR
kc6 = (0.95*tau) / (k*theta);
ti6 = 2.375*theta;
td6 = 0.421*theta;
%Ganancias
Kp6 = kc6;
Ki6 = kc6/ti6;
Kd6 = kc6*td6;

%% Método 3 CHR
% La siguiente tabla con la sintonia prpuesta por el método CHR para el 
% criterio de desempeño "respuesta más rápida posible con 20% de sobreimpulso"
% suponiendo que el sistema es implementado como tipo servo siguiendo una 
% referencia.

% Control P - CHR
kc7 = (0.7*tau) / (k*theta);
%Ganancias
Kp7 = kc7;

% Control PI - CHR
kc8 = (0.6*tau) / (k*theta);
ti8 = tau;
%Ganancias
Kp8 = kc8;
Ki8 = kc8/ti8;

% Control PID - CHR
kc9 = (0.95*tau) / (k*theta);
ti9 = 1.357*tau;
td9 = 0.473*theta;
%Ganancias
Kp9 = kc9;
Ki9 = kc9/ti9;
Kd9 = kc9*td9;

C=tf(kc9*[ti9*td9 ti9 1],[ti9 0]);

